@@ -110,6 +110,13 @@ def compile_Kernels(thr):
     program = thr.compile(open(path/"GPU_Kernels/gaussian_filter.cl").read())
     GPU.gaussian_filter = program.gaussian_filter
 
+    # directCorrelation
+    program = thr.compile(open(path/"GPU_Kernels/directCorrelation.cl").read())
+    GPU.directCorrelation = program.directCorrelation
+
+    program = thr.compile(open(path/"GPU_Kernels/find_peak_direct.cl").read())
+    GPU.find_peak_direct = program.find_peak_direct
+
     thr.synchronize()
 
     return 0
@@ -120,7 +127,7 @@ def initialization(width, height, thr):
     """
 
     # Obtain PIV mesh
-    grid.generate_mesh(width,height)
+    grid.generate_mesh(width, height)
     grid.pixels = width*height
 
     # Total number of boxes for global size
@@ -133,9 +140,8 @@ def initialization(width, height, thr):
     subImg2 = np.zeros((N_boxes_2, Parameters.box_size_2_y,
         Parameters.box_size_2_x), dtype = np.complex64)
 
-
     # Array of parameters data
-    peak_ratio = int(Parameters.peak_ratio*1000)  #trick pass this value as int
+    peak_ratio = int(Parameters.peak_ratio*1000)  #trick to use as int
 
     data1 = np.array((width, height,
         Parameters.box_size_1_x, Parameters.box_size_1_y,
@@ -159,12 +165,23 @@ def initialization(width, height, thr):
     GPU.x2 = thr.to_device(grid.x_2)
     GPU.y2 = thr.to_device(grid.y_2)
 
+    if Parameters.direct_calc:
+        GPU.box_origin_x_d = thr.to_device(grid.box_origin_x_d)
+        GPU.box_origin_y_d = thr.to_device(grid.box_origin_y_d)
+        box_size_x_d = round(Parameters.window_x_1/2)*2+1;
+        box_size_y_d = round(Parameters.window_y_1/2)*2+1;
+        directCorr = np.zeros((N_boxes_1, box_size_y_d,
+            box_size_x_d), dtype = np.float32)
+
+        GPU.size_direct = np.prod(directCorr.shape)
+        GPU.directCorr = thr.to_device(directCorr)
+
     # Send PIV parameters to gpu (only done once)
     GPU.data1 = thr.to_device(data1)
     GPU.data2 = thr.to_device(data2)
     GPU.median_limit = thr.to_device(np.float32(Parameters.median_limit))
 
-    #Temp manager
+    #Temp manager to reduce use of memory
     temp_manager = ZeroOffsetManager(
             thr, pack_on_alloc=True, pack_on_free=False)
 
@@ -273,9 +290,6 @@ def initialization(width, height, thr):
             np.float32, dependencies = [GPU.subImg1_1,
             GPU.subImg1_2, GPU.u_index_1, GPU.v_index_1,
             GPU.img1_g])
-        #img_g = np.float32(np.zeros([height,width]))
-        #GPU.img1_g = thr.empty_like(img_g)
-        #GPU.img2_g = thr.empty_like(img_g)
 
     # Initialice all GPU variables for second iteration (only done once)
     GPU.subImg2_1 = temp_manager.array([N_boxes_2,
@@ -441,6 +455,12 @@ def processing(img1_name, img2_name, thr):
         velocity field in y direction on GPU from second sweep
     """
 
+    if Parameters.weighting:
+        Parameters.weighting = 0
+        print("=======================================================================================")
+        print("Warning: There is a bug in weighting function on the GPU. Parameter.weighting set to 0")
+        print("=======================================================================================")
+
     N_boxes_1 = Parameters.no_boxes_1_x*Parameters.no_boxes_1_y
     N_boxes_2 = Parameters.no_boxes_2_x*Parameters.no_boxes_2_y
     N_pixels_1 = N_boxes_1*Parameters.box_size_1_x*Parameters.box_size_1_y
@@ -481,6 +501,19 @@ def processing(img1_name, img2_name, thr):
                 GPU.data1, local_size = None, global_size = N_pixels_1)
 
     for i in range(0,Parameters.no_iter_1):
+        #First iteration using direct cross correlation if needed
+        if not i and Parameters.direct_calc:
+            #Direct correlation
+            GPU.directCorrelation(GPU.img1, GPU.img2, GPU.directCorr,
+                GPU.box_origin_x_d, GPU.box_origin_y_d, GPU.data1,
+                local_size = None, global_size = int(GPU.size_direct))
+
+            #Find peak
+            GPU.find_peak_direct(GPU.v1, GPU.u1, GPU.directCorr, GPU.data1,
+                local_size = None, global_size = N_boxes_1)
+
+            i += 1
+
         if i:
             # Median Filter
             GPU.Median_Filter(GPU.u1_f, GPU.v1_f, GPU.u1, GPU.v1,
@@ -637,7 +670,6 @@ def processing(img1_name, img2_name, thr):
                     GPU.mask, GPU.data2, local_size = None,
                     global_size = N_boxes_2)
 
-
     # Load Images of next iteration during runtime
     Img1, Img2 = DPIV.load_images(img1_name, img2_name)
 
